******************************************************************************
*                                                                            *
* The MACRO Equations are Declared                                           *
*                                                                            *
******************************************************************************
*GG* V1.0h change the hardwire .001/1000 scaling to us MMSCALE
$ontext
   DD(DM,TP)             markal-macro demand coupling equations
*  Macro submodel:
   UTIL                  utility equation - objective function for MKMC
   MCAP(TP)              capital - trillion dollars
   TMC(TP)               terminal condition  - trillion dollars
   COSTNRG(TP)           cost of energy - billion dollars
   EQ_IVECBND(TP)        bound on sum of investment and energy costs
   EQ_MPEN(TCH, TP)      market penetration
*GG* V1.0f add substitution equation & variables
   EQ_XCAPDB(TP,TCH)     substitution equation for XCAP
$offtext

*=============================================================================*
*  Utility Production Function, the Objective Function                        *
*=============================================================================*
UTIL ..
  SUM(TP,
    DFACT(TP) *
    LOG(
      (
        AKL * (K(TP) ** (KPVS * RHO)) * L(TP) ** ((1 - KPVS) * RHO) +
        SUM(DM, B(DM) * D(DM, TP) ** RHO)
      ) ** (1 / RHO) - IV(TP) - EC(TP)
*GG* V1.0g back out any emissions taxes
*        ( EC(TP) - (SUM(ENV, ENV_COST(ENV, TP) * EM(TP, ENV)))*.000001)
    )
  )
  =E=
      ( (UTILITY / 1000)$(MMSCALE EQ .001) +
       (UTILITY / 1000 * LOG(1000))$(MMSCALE EQ 1) );
*      UTILITY / 1000;
*SK*  UTILITY / (1/MMSCALE);

*=============================================================================*
*  Demand Coupling Equation                                                   *
*  A demand relation is generated for each demand sector DM and ensures that  *
*  the end-use energy output from the demand devices which have output to DM  *
*  is greater than or equal to the end-use demand specified by the user.      *
*=============================================================================*
*GG* V1.1c only generate if demand provided
DD(DM, TP)$(DM_DEMAND(DM, TP) GT 0) ..
  SUM(DMD,
    DEM_CAP(TP, DMD, DM) * CAP(TP, DMD)
  )
  =E=
  (1/MMSCALE) * AEEIFAC(DM, TP) * D(DM, TP) + (1/MMSCALE) * SP(DM, TP) ;


*=============================================================================*
*  Capital Constraint                                                         *
*=============================================================================*
MCAP(TP + 1) ..
  K(TP + 1)
  =E=
*GAMS* 8-3 change 5 to half the number of years-per-period
  K(TP) * TSRV + (NYRSPER/2) * (IV(TP + 1) + IV(TP) * TSRV);

*=============================================================================*
*  Terminal Condition                                                         *
*=============================================================================*
TMC(TLAST) ..
  K(TLAST) * (GROWV(TLAST) + DEPR)/100
  =L=
  IV(TLAST);

*=============================================================================*
*  Cost of Energy                                                             *
*=============================================================================*
*SK* keep factor rather than MMSCALE?
COSTNRG(TP) ..
  0.001 * (

* Resource Supply Activity
    SUM(SEP,
      ANNC_TSEP(TP, SEP) * TSEP(TP, SEP)
    ) +

* Stockpiling of Energy Carriers
    SUM((ENC, P),
      ANNC_ZSTK(TP, ENC, P) * ZSTK(ENC, P)
    ) +

* Activity of Processes
    SUM(PRC,
      ANNC_ACT(TP, PRC) * ACT(TP, PRC)
    ) +

* Capacity of Technologies
    SUM(TCH,
      ANNC_CAP(TP, TCH) * CAP(TP, TCH)
    ) +

* Activity of Electric Plants
    SUM((ELA, Z, Y),
      ANNC_TEZY(TP, ELA, Z, Y) * TEZY(TP, ELA, Z, Y)
    ) +

*GG* V1.1c Activity of Couple Production Plants
    SUM((CPD, Z, Y),
      ANNC_TCZY(TP, CPD, Z, Y) * TCZYH(TP, CPD, Z, Y)
    ) +

* Activity of Heating Plants
*GG* V1.1c add heating plants
    SUM((HPL, Z),
      ANNC_THZ(TP, HPL, Z) * THZ(TP, HPL, Z)
    ) +

* Annualized Investments
    SUM((TC, TCH),
      COST_INV(TP, TC, TCH) * INV(TC, TCH)
    ) +
*SK add the Annualized Investments of subsidized technologies 
*    otherwise the model underestimates the capital investments on energy 
*    when substracts the tax revenue 
    SUM((TC, BEW)$ENV_COST("CO2", TP),
      FRSB*COST_INV(TP, TC, BEW) * INV(TC, BEW)
    ) +

* Annualized costs for RESIDs
*GG* V1.0b FIX the over-accounting on RESIDs!, remove TC
    SUM((TB,TCH)$(TCH_RESID(TCH,TP) NE 0),
      COST_INV(TB, TB, TCH) * RESID(TP, TCH)
    ) +

*GAMS* 8-3 use integer square function to avoid xeq error if XCAP -
*GG* V1.0d add test on QFAC not 0
****************************************************************************
*    SUM(TCH$(1$CAPTB(TCH) AND (QFAC EQ 2)) ,
*      0.5 * QFAC *
*      COST_INV(TP, TP, TCH) * SQR(XCAP(TP, TCH))
*      / (EXPF * CAPTB(TCH))
*    ) +
****************************************************************************
*GG* V1.0f new code subing out SQR(XCAP) with step-wise linear approximation
*          6/12 chosen for quadratic market penetration curve, other
*          values may be better for different shape curves, if appropriate
     (SUM(TCH$(1$CAPTB(TCH) AND 1$TPTCH(TP, TCH)) ,
       0.5 * QFAC *
       COST_INV(TP, TP, TCH) * (CAPTB(TCH) / EXPF *
       (XCAPP(TP,TCH)+6*XCAPP1(TP,TCH)+12*XCAPP2(TP,TCH))) )
     )$(QFAC EQ 1) +

* Emissions Costs
*GG* V1.0g back out any emissions taxes
*GG* V1.4b add MMSCALE for TAX
     SUM(ENV,
*GG* V1.4e use seperate scaler for emissions
*        ENV_COST(ENV, TP) * EM(TP, ENV) / MMSCALE
        ENV_COST(ENV, TP) * EM(TP, ENV) / ENV_SCAL
     ) +

*GG* V1.4e tax/subsidy adjustment
     SUM(TAXSUB,
        TSUB_COST(TAXSUB, TP) * TXSUB(TP, TAXSUB)
     )
  )
  =E=
*GG* V1.0g reduce EC by annual emission_tax_credits, *GG* V1.1c only scale EC
*GG* V1.3c multiply ENV_TAX by MMSCALE, V1.4b always .001 times ENV_TAX
*  (1/MMSCALE) * EC(TP) + SUM(ENV, ENV_TAX(ENV, TP)) * MMSCALE;
*GG* V1.4e add the TSUB_TOT values so that recycled
  (1/MMSCALE) * EC(TP) +
  (SUM(ENV, ENV_TAX(ENV, TP)) + SUM(TAXSUB,TSUB_TOT(TAXSUB,TP))) * .001;

*=============================================================================*
*  Bound on Sum of Investment & Energy                                        *
*=============================================================================*
EQ_IVECBND(TT) ..
  IV(TT) + EC(TT)
  =L=
*GG* V1.0h user scalar (from CONSTANT table) to control investment tolerance
  Y0 * L(TT) ** IVETOL;

*=============================================================================*
*  Market Penetration Cost Penalty Function                                   *
*=============================================================================*
*GG* V1.0f add test on QFAC not 0 and costs provided
EQ_MPEN(TCH, TP + 1)$(1$CAPTB(TCH) AND (QFAC NE 0) AND (1$TPTCH(TP,TCH))
                      AND (1$COST_INV(TP+1, TP+1, TCH)) ) ..
  CAP(TP + 1, TCH)
  =L=
  EXPF * CAP(TP, TCH) + XCAP(TP + 1, TCH);

*=============================================================================*
*  Market Penetration Cost Penalty Function, Quadratic Approximation          *
*=============================================================================*
*GG* V1.0f add equation for approximation of quad market penalty
EQ_XCAPDB(TP+1,TCH)$(1$CAPTB(TCH) AND (QFAC EQ 1) AND (1$TPTCH(TP,TCH))
                      AND (1$COST_INV(TP+1, TP+1, TCH)) ) ..
  XCAP(TP + 1, TCH)
  =E=
  XCAPP(TP + 1, TCH) + XCAPP1(TP+1, TCH) + XCAPP2(TP+1, TCH);

